package com.diagbot.facade;

import com.diagbot.dto.DictionaryInfoDTO;
import com.diagbot.enums.RedisEnum;
import com.diagbot.enums.StandConvertEnum;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.RedisUtil;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author:zhoutg
 * @time: 2018/11/23 11:37
 */
@Component
public class CacheFacade {

    @Autowired
    RedisUtil redisUtil;
    @Autowired
    NeoFacade neoFacade;
    @Autowired
    private KlDiagnoseBaseFacade klDiagnoseBaseFacade;
    @Autowired
    KlDictionaryInfoFacade klDictionaryInfoFacade;

    /**
     * 清除缓存信息
     *
     * @return
     */
    @CacheEvict(value = "cache", allEntries = true)
    public void clear() {

    }

    /**
     * 清除标准词转换缓存信息
     *
     * @return
     */
    public void clearStandConvert() {
        redisUtil.deleteByPrex(StandConvertEnum.disease.getName());
        redisUtil.deleteByPrex(StandConvertEnum.drug.getName());
        redisUtil.deleteByPrex(StandConvertEnum.lis.getName());
        redisUtil.deleteByPrex(StandConvertEnum.operation.getName());
        redisUtil.deleteByPrex(StandConvertEnum.pacs.getName());
        redisUtil.deleteByPrex(StandConvertEnum.symptom.getName());
        redisUtil.deleteByPrex(StandConvertEnum.vital.getName());
        redisUtil.deleteByPrex(StandConvertEnum.transfusion.getName());
    }

    /**
     * 加载药品类型缓存
     *
     * @return
     */
    public void loadDrugTypeCache() {
        redisUtil.deleteByPrex(RedisEnum.drugType.getName());
        neoFacade.getDrugCache();

    }

    /**
     * 加载数据库中症状特征缓存
     *
     * @return
     */
    public void getSymptomFeatureCache() {
        redisUtil.deleteByPrex(RedisEnum.symptomFeature.getName());
        neoFacade.symptomFeatureCache();
    }

    /**
     * 加载数据库中疾病对应缓存
     *
     * @return
     */
    public void getDiseaseCorrespondCache() {
        redisUtil.deleteByPrex(RedisEnum.diseaseCorrespond.getName());
        neoFacade.diseaseCorrespondCache();
    }

    /**
     * 加载数据库中疾病过滤缓存
     *
     * @return
     */
    public void getdiseaseFilterCache() {
        redisUtil.deleteByPrex(RedisEnum.diseaseFilter.getName());
        neoFacade.diseaseFilterCache();
    }

    /**
     * 缓存医学知识库中的所有规则
     *
     * @return
     */
    public void loadAllRuleCache(){
        redisUtil.deleteByPrex(RedisEnum.allRule.getName());
        neoFacade.allRuleCache();
    }

    /**
     * 缓存医学知识库中的所有去重的基础诊断依据
     *
     * @return
     */
    public void loadAllBaseDiagnoseCache(){
        redisUtil.delete(RedisEnum.allBaseDiagnose.getName());
        neoFacade.allBaseDiagnose();
    }

    public void clearLoadCache(String str) {
        redisUtil.deleteByPrex(str);
    }

    /**
     * 加载医院大数据推送类型
     */
    public void loadHostipalPushType() {
        redisUtil.delete(RedisEnum.hospitalPushType.getName());
        redisUtil.delete(RedisEnum.pushVersion.getName());
        List<DictionaryInfoDTO> listByGroupType = klDictionaryInfoFacade.getListByGroupType(Lists.newArrayList(100,101));
        if (ListUtil.isNotEmpty(listByGroupType)) {
            Map<Long, List<DictionaryInfoDTO>> map = EntityUtil.makeEntityListMap(listByGroupType, "groupType");
            if (map.get(Long.valueOf(100)) != null) {
                redisUtil.putHashMap(RedisEnum.hospitalPushType.getName(),
                        map.get(Long.valueOf(100)).stream().collect(Collectors.toMap(k -> k.getName(), v -> v.getVal(), (v1, v2) -> (v2))));
            }
            if (map.get(Long.valueOf(101)) != null) {
                redisUtil.putHashMap(RedisEnum.pushVersion.getName(),
                        map.get(Long.valueOf(101)).stream().collect(Collectors.toMap(k -> k.getName(), v -> v.getVal(), (v1, v2) -> (v2))));
            }
        }
    }
}
